<?php
/* Copyright start
  Copyright (C) 2008 - 2024 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Providers;

use App\Entity\Query\Query;
use Psr\Log\LoggerInterface;
use App\Doctrine\ORM\FilterQueryBuilder;
use Doctrine\Persistence\ManagerRegistry;
use Doctrine\ORM\Mapping\ClassMetadataInfo;
use ApiPlatform\Core\DataProvider\Pagination;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use App\Providers\AggregateDataProviderInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;
use ApiPlatform\Core\Bridge\Doctrine\Orm\ItemDataProvider;

class AggregateDataProvider extends ItemDataProvider implements AggregateDataProviderInterface
{

    public function __construct(
        private ManagerRegistry $managerRegistry,
        private FilterQueryBuilder $filterQueryBuilder,
        private Pagination $pagination,
        private LoggerInterface $logger
    ) {}

    public function getAggregate(string $resource, Request $request, $query)
    {
        $entityClass = $resource; //->getEntityClass();

        /** @var EntityManager $manager */
        $manager = $this->managerRegistry->getManagerForClass($entityClass);
        $repository = $manager->getRepository($entityClass);

        $page = (int) $request->get($this->pagination->getPage(), 1);

        $itemsPerPage = $this->pagination->getLimit();
        if ($requestedItemsPerPage = $request->get('$limit')) {
            $itemsPerPage = (int) $requestedItemsPerPage;
        }

        $queryBuilder = $repository
            ->createQueryBuilder('o')
            ->setFirstResult(($page - 1) * $itemsPerPage)
            ->setMaxResults($itemsPerPage);

        if (!(isset($query['logic']) && isset($query['filters']))) {
            $query['logic'] = "AND";
            $query['filters'] = [];
        }

        $returnField = ["o.uuid"];
        $queryClone = $query;

        if ($query && array_key_exists(Query::KEYWORD_AGGREGATES, $query)) {
            unset($query[Query::KEYWORD_AGGREGATES]);
        }
        $this->filterQueryBuilder->fromQueryObject($query, $queryBuilder, $entityClass, $returnField, false);


        $classMetaData = $manager->getClassMetadata($entityClass);
        $identifiers = $classMetaData->getIdentifier();

        foreach ($classMetaData->getAssociationNames() as $i => $association) {
            $mapping = $classMetaData->associationMappings[$association];

            if (ClassMetadataInfo::FETCH_EAGER === $mapping['fetch']) {
                $queryBuilder->leftJoin('o.' . $association, 'a' . $i);
                $queryBuilder->addSelect('a' . $i);
            }
        }

        if (isset($this->order) && 1 === (is_countable($identifiers) ? count($identifiers) : 0)) {
            $identifier = $identifiers[0];
            $queryBuilder->addOrderBy('o.' . $identifier, $this->order);
        }

        /*
        $queryBuilder will give us the query with all the where clauses including filter requested via API call.
        But will only be returning uuid as part of response as we have explicitly specified it in $returnFields
        This will be used as subQuery.

        $qb here a second instance of query Builder needs to be used to form a parent query.
        Here as part of our query we are mainly passing aggregation operation, so it will only return the aggregation expression in the form of select.

        In line 98 we are forming a main and sub-query to get us the desired result of aggregation.
        Mantis: 1060002
        */
        $qb = $repository
            ->createQueryBuilder('d')
            ->setFirstResult(($page - 1) * $itemsPerPage)
            ->setMaxResults($itemsPerPage);
        $this->filterQueryBuilder->fromQueryObject($queryClone, $qb, $entityClass, queryAlias: 'd');
        $qb->setParameters(($queryBuilder->getParameters()));
        try {
            return $qb->where($qb->expr()->in('d.uuid', $queryBuilder->getDql()))->getQuery()->execute();
        } catch (\Throwable $th) {
            $errMsg = $th->getMessage();
            throw new HttpException(Response::HTTP_BAD_REQUEST, $errMsg);
        } catch (\Exception $e) {
            $errMsg = $e->getMessage();
            throw new HttpException(Response::HTTP_BAD_REQUEST, $errMsg);
        }
    }
}
